#!/usr/bin/perl

use strict;
use warnings;

use Bio::SeqIO;

my $finallist = $ARGV[0];
my %ltaggennamehash = ();
my $ltag = "";
my @gbklines = ();
my $genname = "";
my $note = "";
my $genid = "";
my @ids = ();
my $columncount = 1;
my %ltagcolumnhash = ();
my %ltaggenidhash = ();
my $argofinterestswitch = 1;

print "p-value,";

for(my $i=1;$i<=(scalar(@ARGV)-1);$i++) {
    $columncount++;
    my @splitarg = split(/,/, $ARGV[$i]);
    if ($splitarg[0] =~ m/(NC_\d{6})/) {
        my $temp = $1;
        chomp $temp;
        print "$temp,";
    }
    foreach (@splitarg) {

my $seqio_object = Bio::SeqIO->new(-file => $_);
my $seq_object = $seqio_object->next_seq;

for my $feat_object ($seq_object->get_SeqFeatures) {
   push @ids, $feat_object->get_tag_values("locus_tag") if ($feat_object->has_tag("locus_tag"));
   if (exists $ids[0]) {
   } else { $ids[0] = "N/A"; }
   push @ids, $feat_object->get_tag_values("gene") if ($feat_object->has_tag("gene"));
   if (exists $ids[1]) {
   } else { $ids[1] = "N/A"; }
   push @ids, $feat_object->get_tag_values("db_xref") if ($feat_object->has_tag("db_xref"));
   if (exists $ids[2]) {
   } else { $ids[2] = "N/A"; }


   if (exists $ids[0]) {
       $ltag = lc($ids[0]);
   } else {
       $ltag = "N/A";
   }
   if (exists $ids[1]) {
       $genname = $ids[1];
   } else {
       $genname = "N/A";
   }
   if (exists $ids[2]) {
       for(my $i=2; $i < scalar(@ids); $i++) {
           if ($ids[$i] =~ m/GeneID:/) {
               $genid = $ids[$i];
           }
       }
   } else {
       $genid = "N/A";
   }

   chomp $ltag;
   chomp $genname;
   chomp $genid;
   if($ltag ne "N/A") {
       $ltagcolumnhash{$ltag} = $columncount; 
   }
   if ($ltag ne "N/A" and $genname ne "N/A") {
        $ltaggennamehash{$ltag} = $genname;
   }
   if ($ltag ne "N/A" and $genid ne "N/A" and $genid =~ m/GeneID:/) {
        $ltaggenidhash{$ltag} = $genid;
   }
   $ltag = "";
   $genname = "";
   $genid = "";
   @ids = ();
}
}
$argofinterestswitch = 0;
}

print "Annotation";

print "\n";

open(MYDATA, $finallist) or die("Error: cannot open file" . $finallist . "\n");

my @finallistlines = <MYDATA>;

close MYDATA;


my @files = ();
@files = <*>;
my %pvalenergyhash = ();
my $pvalenergy = '';

foreach (@files) {
    if ($_ =~ m/\.final\.csv/) {
        open (DATA, $_) or die ("Error: cannot open file" . $_ . "\n");
            my @datalines = <DATA>;
        close DATA;
    foreach my $line (@datalines) {
        my @splitarg = split(/;/, $line);
        foreach(@splitarg) { chomp $_; }
        $pvalenergy = "|" . $splitarg[6] . "|" . $splitarg[7] . "|" . $splitarg [1] . "|" . $splitarg [2] . "|" . $splitarg [4] . "|" . $splitarg [5];
        chomp $pvalenergy;
        $pvalenergyhash{lc($splitarg[0])} = $pvalenergy;
    }
    }
}


my $clustertab = "cluster.tab";

open(MYDATA, $clustertab) or die("Error: cannot open file" . $clustertab . "\n");

my @clustertablines = <MYDATA>;

close MYDATA;

my %annotationhash = (); #locustags -> annotation

my @splitScndArgv = split(/,/,$ARGV[1]);

foreach (@splitScndArgv) {
    my $in  = Bio::SeqIO->new(-file => $_ , '-format' => 'genbank');
    while ( my $seq = $in->next_seq() ) {
        foreach my $sf ( $seq->get_SeqFeatures() ) {
            
            # add exception for "hypothetical protein" annotation
            my $product = "";
            my @products = ();
            if ($sf->has_tag("locus_tag") and $sf->has_tag("product")) {
                @products = $sf->get_tag_values("product");
                $product = $products[0];
            }

            if ($sf->has_tag("locus_tag") and $sf->has_tag("product") and (not $product =~ m/hypothetical protein/)) { # product
                my @ltaglist = $sf->get_tag_values("locus_tag");
                my $ltag = $ltaglist[0];
                my @prodlist = $sf->get_tag_values("product");
                my $prod = $prodlist[0];
                $annotationhash{$ltag} = $prod;
            } 
            elsif ($sf->has_tag("locus_tag") and $sf->has_tag("note")) { # note
                my @ltaglist = $sf->get_tag_values("locus_tag");
                my $ltag = $ltaglist[0];
                my @notelist = $sf->get_tag_values("note");
                my $note = $notelist[0];
                $annotationhash{$ltag} = $note;
            }
            elsif ($sf->has_tag("locus_tag") and $sf->has_tag("function")) { # function
                my @ltaglist = $sf->get_tag_values("locus_tag");
                my $ltag = $ltaglist[0];
                my @funclist = $sf->get_tag_values("function");
                my $func = $funclist[0];
                $annotationhash{$ltag} = $func;
            }
        }
    }    
}

foreach (@finallistlines) {
    my $commacount = 1;
    my @splitlines = split(/;/, $_);
    pop(@splitlines);
    print $splitlines[0] . ",";
    shift(@splitlines);
    my @newsplit = ();
    my @newsplit2 = ();
    
    foreach (@splitlines) { 
       $newsplit[$ltagcolumnhash{$_}] = $_; 
    } 
 
    foreach (@newsplit) {
       if (defined $_) {
       push (@newsplit2, $_);
       }
    }

  

    my $nscount = 0;
    if ($ltagcolumnhash{$newsplit2[0]} ne 2) {

        for(my $k=2; $k<$ltagcolumnhash{$newsplit2[0]};$k++) {
            $commacount++;
            print ",";
        }
    }    

    foreach(@newsplit2) {
        print $newsplit2[$nscount];
        if (exists $ltaggennamehash{$newsplit2[$nscount]}) { # this is the locus tag

            print "(" . $ltaggennamehash{$newsplit2[$nscount]} . $pvalenergyhash{$newsplit2[$nscount]} . "|";
        
        } else {

            print "(N/A" . $pvalenergyhash{$newsplit2[$nscount]} . "|";
        }
       
        if (exists $ltaggenidhash{$newsplit2[$nscount]}) { # this is the gene id
            print $ltaggenidhash{$newsplit2[$nscount]} . ")";
        } else {
            print "N/A)";
        }

        if (exists $newsplit2[$nscount + 1]) {
        for (my $j=$ltagcolumnhash{$newsplit2[$nscount]}; $j<$ltagcolumnhash{$newsplit2[$nscount + 1]};$j++) {
            print ",";
            $commacount++;
        }
        }
 
$nscount++;
}
# print annotation
for(my $i=$commacount; $i < scalar(@ARGV); $i++) {
        print ",";
}

foreach my $key (keys %annotationhash) {
    my $thetag = $newsplit2[0];
    my $temp = $annotationhash{$key};
    if ($key =~ m/$thetag/i) {
        my $omitcomma = $annotationhash{$key}; 
        $omitcomma =~ s/,/ /g;
        chomp $omitcomma;
        print $omitcomma;
        last;
    } 
}
                                       
print "\n";  
}

